#!/bin/bash

# Packager : Jean-Philippe Guillemin <jp.guillemin@free.fr>
# Sources for gdm : http://ftp.gnome.org/pub/GNOME/sources/gdm/
# Sources for numlockx : http://ktown.kde.org/~seli/numlockx/


softversion='2.20.11'
packageversion='72.1'
softname='gdm'
arch='i486'
cpu='i686'

buildir=$(pwd)

srcpkg="$buildir/$softname-$softversion.tar.gz"
src="$buildir/$softname-$softversion"

package="$softname-$softversion-$arch-$packageversion"

dest="/tmp/$package"


cd $buildir
rm -rf $src
tar -xvf $srcpkg
rm -f $dest.tgz
rm -rf $dest
mkdir -p $dest
chown -R root:root .
export CFLAGS="-O2 -march=$arch -mtune=$cpu"
#export PKG_CONFIG_PATH=/usr/lib/pkgconfig

cd $src
./configure \
  --prefix=/usr \
  --sysconfdir=/etc/X11 \
  --mandir=/usr/man \
  --with-prefetch=yes \
  --localstatedir=/var/state \
  --enable-static=no \
  --disable-debug \
  --with-defaults-conf=/etc/X11/gdm/defaults.conf \
  --with-dbus-sys=/etc/dbus-1/system.d \
  --with-gconf-schema-file-dir=/etc/gconf/schemas \
  --with-pam-prefix=/etc

make -j2
make install DESTDIR=$dest
rm -f $dest/usr/info/dir

gzip -9 $dest/usr/man/man?/*

mkdir -p $dest/usr/doc/$softname-$softversion
cp -a AUTHORS COPYING INSTALL NEWS README README.install TODO $dest/usr/doc/$softname-$softversion/
chmod -R 644 $dest/usr/doc/$softname-$softversion/*

mkdir -p $dest/usr/src/$softname-$softversion
cp $buildir/build-$softname.sh $dest/usr/src/$softname-$softversion/

# numlockx setup
cd $buildir
rm -rf numlockx-1.1
tar -xvf numlockx-1.1.tar.gz
(
cd numlockx-1.1
./configure --prefix=/usr --x-includes=/usr/include/X11 --x-libraries=/usr/lib

make
make install DESTDIR=$dest

mkdir -p $dest/usr/doc/numlockx-1.1
cp -a AUTHORS ChangeLog LICENSE INSTALL README TODO $dest/usr/doc/numlockx-1.1
chown -R root:root $dest/usr/doc/numlockx-1.1
)
rm -rf  $buildir/numlockx-1.1

sed -i '/^exit 0.*$/d' $dest/etc/X11/gdm/Init/Default
cat << "EOF" >> $dest/etc/X11/gdm/Init/Default
if [ -e /etc/rc.d/rc.numlock ] ; then
	[ -x /etc/rc.d/rc.numlock ] && /usr/bin/numlockx on
else
	/usr/bin/numlockx on
fi

EOF

# END numlockx setup

( 
  cd $dest
  find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

cd $buildir

# Gdm paths
mkdir -p $dest/var/log/gdm
mkdir -p $dest/var/lib/gdm
chown -R gdm.gdm $dest/var/lib/gdm
chmod 1770 $dest/var/lib/gdm

# PAM configuration
mkdir -p $dest/etc/pam.d
rm -f $dest/etc/pam.d/*

cat << "EOF" > $dest/etc/pam.d/gdm
auth        required    pam_env.so
auth		include		system-auth
account		required	pam_nologin.so
account		include		system-auth
password	include		system-auth
session	    required	pam_unix.so
session     required    pam_limits.so
session		required	pam_loginuid.so
session     optional    pam_keyinit.so force revoke
session     required    pam_namespace.so
EOF

cat << "EOF" > $dest/etc/pam.d/gdm-autologin
auth       required    pam_env.so
auth       required    pam_permit.so
account    required    pam_nologin.so
account    include     system-auth
password   include     system-auth
session    required    pam_loginuid.so
session    optional    pam_keyinit.so force revoke
session    required    pam_namespace.so
session    include     system-auth
EOF

# Prefetched libs
rm -f $buildir/temp.txt
for binary in /usr/bin/thunar /usr/bin/xfwm4 /usr/bin/xfce4-panel /usr/lib/icecat/icecat-bin ; do
	ldd $binary | sed -e 's/.*=>[ \t]*\([^ \t]*\)[ \t]*(.*/\1/' | egrep "xfce|gtk|gdk|dbus|nss|nspr|cairo|moz|xpcom|nspr" >> $buildir/temp.txt
done
sort -u $buildir/temp.txt > $dest/etc/X11/gdm/gdmprefetchlist


# Improved Xsession
cat $buildir/Xsession > $dest/etc/X11/gdm/Xsession
chmod 755 $dest/etc/X11/gdm/Xsession
rm $dest/usr/share/gdm/BuiltInSessions/default.desktop

# Default config
chmod 644 $buildir/defaults.conf
cp -pf $buildir/defaults.conf $dest/etc/X11/gdm/defaults.conf
mv $dest/etc/X11/gdm/custom.conf $dest/etc/X11/gdm/custom.conf.new

# Theming
rm -rf $dest/usr/share/gdm/themes
cp -rf $buildir/themes $dest/usr/share/gdm/themes
mkdir -p $dest/usr/share/sounds
cp -rf $buildir/Zen $dest/usr/share/sounds

# Icons
rm -rf $dest/usr/share/icons/hicolor
mkdir -p $dest/usr/share/icons/hicolor/scalable/apps
cp -f $buildir/icons/* $dest/usr/share/icons/hicolor/scalable/apps

# Launchers
mkdir -p $dest/usr/share/applications
mv $dest/usr/share/gdm/applications/* $dest/usr/share/applications
rm -rf $dest/usr/share/gdm/applications
sed -i 's/^Exec=.*/Exec=gksu gdmsetup/g'  $dest/usr/share/applications/gdmsetup.desktop
sed -i 's/Categories=.*/Categories=GNOME;GTK;Settings;System;X-Zenpanel;X-XFCE-SettingsDialog;X-XFCE-PersonalSettings;/g'  $dest/usr/share/applications/gdmsetup.desktop
sed -i '/TryExec/d' $dest/usr/share/applications/gdmsetup.desktop

mkdir -p $dest/install

cat << "EOF" > $dest/install/slack-desc
gdm: GDM - the GNU Display Manager
gdm:
gdm: The GNU Display Manager is a reimplementation of the well known
gdm: xdm program.  gdm implements all significant features required for
gdm: managing local and remote displays.
gdm:
gdm:
gdm:
gdm:
gdm:
gdm:
EOF


cat << "EOF" >> $dest/install/doinst.sh
#!/bin/sh

( cd usr/bin ; rm -rf gdm )
( cd usr/bin ; ln -sf /usr/sbin/gdm gdm )

# Handle the incoming configuration files:
config() {
  NEW="$1"
  OLD="`dirname $NEW`/`basename $NEW .new`"
  # If there's no config file by that name, mv it over:
  if [ ! -r $OLD ]; then
    mv $NEW $OLD
  elif [ "`cat $OLD | md5sum`" = "`cat $NEW | md5sum`" ]; then # toss the redundant copy
    rm $NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}
config etc/X11/gdm/custom.conf.new

EOF


cd $dest
makepkg  -l y -c n $dest.txz

# cleanup
rm -rf $dest
rm -rf $src


